@@ -4,20 +4,32 @@ use strict;
 use warnings;
 use 5.014;
 
+our $VERSION = 0.05;
+
 use Tie::RegexpHash 0.16;
 use List::Util qw(any);
 
 use Exporter 'import';
 use base qw/Bot::BasicBot/;
 
-our @EXPORT_OK = qw(command);
+our @EXPORT_OK = qw(command autocommand);
 
 my %command;
+my %autocommand;
 
 sub command {
     (caller)[0]->declare_command(@_);
 }
 
+sub autocommand {
+    (caller)[0]->declare_autocommand(@_);
+}
+
+sub declare_autocommand {
+    my ($package, $sub) = @_;
+    $autocommand{$package} = $sub;
+}
+
 sub declare_command {
     my $package = shift;
     my $sub = pop;
@@ -45,10 +57,12 @@ sub new {
 
     my $address = delete $opts{address};
     my $trigger = delete $opts{trigger};
+    my $bark    = delete $opts{bark} // 1;
 
     my $self = $class->SUPER::new(%opts);
     $self->{trigger} = $trigger if defined $trigger;
     $self->{address} = $address if defined $address;
+    $self->{bark}    = $bark;
 
     return $self;
 }
@@ -57,26 +71,34 @@ sub said {
     my $self = shift;
     my ($data) = @_;
 
+    $self->{command} = $data;
     my $package = ref $self;
 
-    if (my @auto = $self->_auto($data->{body}) {
-        return join " ", @auto;
-    }
+    my ($cmd, $message) = split ' ', $data->{body}, 2;
+
+    my $autosay = $self->_auto($data->{body});
 
     if ($self->{address} and not $data->{address}) {
-        return;
+        return $autosay;
     }
 
     if ($self->{trigger} and $data->{body} !~ s/^\Q$self->{trigger}//) {
-        return;
+        return $autosay;
     }
 
-    my ($cmd, $message) = split ' ', $data->{body}, 2;
-    my $found = $command{$package}{$cmd} or return "What is $cmd?";
+    my $found = $command{$package}{$cmd};
 
-    any { $_ eq 'said' } @{$found->{events}} or return;
+    if (!$found) {
+        return "What is $cmd?" if $self->{bark};
+        return $autosay;
+    }
 
-    $found->{sub}->($self, $cmd, $message);
+    my $say;
+    if (any { $_ eq 'said' } @{$found->{events}}) {
+        $say = $found->{sub}->($self, $cmd, $message);
+    }
+
+    return $say // $autosay;
 }
 
 sub emoted {
@@ -85,7 +107,10 @@ sub emoted {
 sub noticed {
 }
 
-sub _auto { }
+sub _auto {
+    my ($self, $message) = @_;
+    join ' ', map $_->($self, $message), values %autocommand;
+}
 
 1;
 
@@ -118,10 +143,16 @@ Simple declarative syntax for an IRC bot that responds to commands.
 
 Construction of the bot is the same as Bot::BasicBot, as is running it.
 
-CommandBot takes two new options to C<new>:
+CommandBot takes three new options to C<new>:
 
 =over
 
+=item C<bark>
+
+If true, the bot will ask "What is $cmd?" for any command for which no
+appropriate handler is found. If not, the bot will remain silent. This defaults
+to true.
+
 =item C<trigger>
 
 If provided, this string will be required at the start of any message for it to
@@ -137,7 +168,7 @@ addressed, nothing will happen.
 
 =back
 
-Despite the above, the C<_auto> method will always be called, regardless.
+Despite the above, autocommands will always be called, regardless.
 
 If both options are provided, the bot must be addressed I<and> the command must
 be prefixed with the trigger for a response to happen.
@@ -151,8 +182,8 @@ after the preprocessing done by the address and trigger detection.
     Commandbot: command text text
     Commandbot: !command text text
 
-In all these cases, C<command> is the B<command>. C<text text> is then a single string, regardless of how long it is.
-This is the B<message>.
+In all these cases, C<command> is the B<command>. C<text text> is then a single
+string, regardless of how long it is.  This is the B<message>.
 
 The command string is then looked up in the list of declared commands. If it is
 exactly equal to a command declared as a string, or matches a command declared
@@ -165,7 +196,7 @@ If it does not match, the bot says "What is $command?".
 =head2 command
 
 The C<command> function declares a command. It accepts either a string or a
-regex and a subref. The subref will be called whenever the bot is activated
+regex, and a subref. The subref will be called whenever the bot is activated
 with a matching string.
 
 The subref receives C<$self>, C<$cmd> and C<$message>: The bot object, the
@@ -191,16 +222,31 @@ same arguments as C<command>, but it is called on a package:
 This can be helpful if you don't want to put all your commands in the same
 module - you can declare them all on the same package.
 
-=head2 _auto
+    sub import {
+        my $caller = (caller)[0];
+        $caller->declare_command(...);
+    }
 
-C<_auto> is a plain method on your bot. It is called before anything else is
-done, and if it returns anything, that is said and nothing else is done.
+=head2 autocommand
 
-As mentioned this is a normal method, so it receives C<$self> first, and then
-C<$message>.  There is no C<$cmd> because no command was involved.
+An autocommand will be processed regardless of whether the message was
+interpreted as a command or not.
 
-This sub is intended as a hook for you to perform any actions necessary as a
-result of people talking in general, or for bots that think they're human and
-want to join in.
+    autocommand => sub {
+        my $self = shift;
+        my $message = shift;
+    };
 
-It is called in list context, so remember not to return undef.
+This is intended as a hook for you to perform any actions necessary as a result
+of people talking in general, or for bots that think they're human and want to
+join in.
+
+It is not generally considered sensible to return a value from autocommands,
+but if you wish to, be aware that the return value of autocommands is only
+spoken if an actual command does not say something. Also, the return values
+from all autocommands across all packages will be joined together
+
+=head2 declare_autocommand
+
+The equivalent of C<declare_command> for autocommands. Lets you create an
+autocommand from a package besides one that extends the bot.
